import string
import sys
import csv
import pandas as pd
import math
import numpy as np

'''README:
This file is structured such that the functions are invoked in a bottom-up order (start at the bottom).
Wherever there is a <??#??>, you need to complete the code in this space. 
If you have a better method, please feel free to use it (but you must produce the correct Data warehouse)
Once your code is complete you can call this script to turn your three data sets into a consolidated warehouse.
Try commenting out calls to functions to debug components of this script one at a time.
Use whatever python interpreter you choose, e.g. PyCharm (I just use CMD)
'''

def syntheticClassVars(DW):
 
  #label each employee as Top, Mid, or Low-Tier Talent according to the following rules: Low [0-2.5), Mid [2.5-3.5), Top [3.5-5]
  TTRules=lambda row: 'Top' if (<??R??>) else ('Mid' if (<??S??>) else 'Low')
  
  '''This alternative labelling method uses 4 classes instead of 3 (not required but included for reference)'''
  #label each employee as Top, Mid, Low, or Bottom-Tier Talent according to the following rules: Low [0-2.5), Mid [2.5-3.5), Top [3.5-5]
  #TTRules=lambda row: 'Top' if (row['PEAverage']>=3.5) else ('Mid' if (row['PEAverage']>=2.5) else ('Low' if row['PEAverage']>=1.5 else 'Bottom'))
  
  DW.insert(2, 'TalentTier', DW.apply(<??T??>,axis=1)) #insert the new TalentTier column in position 2
  
  return DW

  
def syntheticVars(DW):

  #get simplified date variables (the day won't be helpful, date format not handled well)
  idx = DW.columns.get_loc("HireDate")
  DW.insert(idx+1, 'HireMonth', DW['HireDate'].apply(lambda x: pd.Timestamp(x).month))
  DW.insert(idx+2, 'HireYear', DW['HireDate'].apply(lambda x: pd.Timestamp(x).year))
  
  #get generation from age
  '''Bonus: create a "generation" column based on birth year rules: Millennial >1980, GenX [1980-1965], BabyBoomer <1965. Insert this column next to the Age attribute.''''


  return DW 


def syntheticPEVar(DW):

  #get index of column you want to insert next to
  idx = DW.columns.get_loc("PE2015")+1

  #insert empty synthetic variable column called PEAverage
  DW.insert(idx, 'PEAverage', ['']*len(DW))
  
  for i in range(len(DW)):
    PE = np.array([DW.PE2012[i], DW.PE2013[i], DW.PE2014[i], DW.PE2015[i]]) #create a numpy array so we can perform calculations across the array
    DW.PEAverage[i] = np.nanmean(<??Q??>) #Get non-nan average performance over 4 years
    
  '''Bonus: perform this calculation without a loop using the 'lambda row:' function'''

  return DW  

def alterVariables(DW):

  '''We don't need this step with our data set.
  e.g. we could use this function if we had sparse data with high precision decimals,  
  in this case it might be better to alter the column to be a T/F attribute'''

  return DW

def consolidate(HRData, PayData, PerfEvals):

  #bring HRData and PayData together
  HRandPay = pd.merge(<??K??>, PayData, how='<??L??>', on='<??M??>')

  #bring HRData, PayData, and PerfEvals together to make the data warehouse
  DW = pd.merge(<??N??>, PerfEvals, how='<??O??>', on='<??P??>')

  return DW


def transfPerfEvals(PerfEvals,IDs):
  '''use pandas merge with full set of employee IDs to transform row data to column data.''' 
  
  #initialize new data structure as an empty data frame with one row per employee
  NewPerfEvals = IDs
  
  #create an empty dictionary corresponding to the 4 years of data
  PerfYears={2012: '',2013: '',2014: '',2015: ''}

  #add performance review data to NewPerfEvals data frame

  for y in range(2012,<??E??>): #iterate through the 4 years, creating a temp data frame for each year
    PerfYears[y] = PerfEvals[(PerfEvals['<??F??>'] == int(y))] #get all rows from PerfEvals where year matches 'y'
    PerfYears[y] = PerfYears[y][['<??G??>','<??H??>']] #just keep ID and Eval - don't need year any more
    PerfYears[y].columns = ['EmployeeID', 'PE%s' % y] #rename the evaluation column to indicate the year
    #merge the new column into the NewPerfEvals data frame by matching the Emp IDs
    '''Think: how should we merge? Inner, Outer, Right, or Left? What happens if employee X isn't employed in year Y?'''
    NewPerfEvals = pd.merge(NewPerfEvals, PerfYears[y], how='<??I??>', on='<??J??>')
  
  return NewPerfEvals


def readTables(filename):
  
  #read in all tables from the raw HR data file
  xls = pd.ExcelFile(filename) #initialize pandas excel reader
  HRData = xls.parse('<??B??>') #this is now a pandas data frame; pandas operations apply to "HRData"
  PayData = xls.parse('<??C??>')
  PerfEvals = xls.parse('<??D??>')
    
  #rename the type column in HRData to be less ambiguous; not have spaces in the names (allows us to use dot-operations on columns)
  HRData.rename(columns={'Employee Name':'Name', 'Hire Date': 'HireDate', 'Type':'EmpType'},inplace=True) #just a few examples, the rest are fine
  
  #get list of Employee IDs as "spine" for new data frames
  IDs = HRData.copy(deep=True) #make a copy of the HRData DF
  IDs = IDs[['EmployeeID']] #Drop all columns from this new DF except EmployeeID
  '''note: There are a number of ways of doing this, but this way keeps the DF structure; simply copying the column turns it into a vector'''

  #transform Performance Evaluation data to align with HR Data (need each row to be one employee in DW)
  PerfEvals = transfPerfEvals(PerfEvals,IDs)

  #consolidate the three tables into a single table (using Employee ID as the key)
  allData = consolidate(HRData, PayData, PerfEvals)

  return allData

  
def main():
  #read filename and call custom readTables function to read in the worksheets, and transform and consolidate them
  filename = '<??A??>'
  data = readTables(filename)

  #alter some of the default variables to be more useful
  data = alterVariables(data)

  #create synthetic average PE variable from the default PE variables
  data = syntheticPEVar(data)
  
  #create synthetic date variables from the default date variables
  data = syntheticVars(data)
  
  #create synthetic class variables
  data = syntheticClassVars(data)
  
  #print all the data to a csv
  data.to_csv('HRDataWarehouse.csv',index=False)
  
  
# Boilerplate to launch main()
if __name__ == '__main__':
    main()
